#include "MainComponent.h"

class TrackSegmentSource : public juce::PositionableAudioSource {
public:
    TrackSegmentSource() {
        juce::CoreAudioFormat coreAudioFormat;
        
        auto stream = std::make_unique<juce::MemoryInputStream>(BinaryData::_g8nAqDu3gIa_mp4,
                                                                BinaryData::_g8nAqDu3gIa_mp4Size,
                                                                true);
        
        if (auto reader = coreAudioFormat.createReaderFor (stream.release(), true)) {
            sourceSampleRate = reader->sampleRate;
            
            totalRange = juce::Range<juce::int64>::withStartAndLength (0, reader->lengthInSamples);
            targetRange = juce::Range<juce::int64>::withStartAndLength (0, reader->lengthInSamples);
            
            sourceRange = juce::Range<juce::int64>::withStartAndLength (sourceSampleRate * 7,
                                                                        sourceSampleRate * 30);
            
            subsectionReader = std::make_unique<juce::AudioSubsectionReader> (reader,
                                                                              sourceRange.getStart(),
                                                                              sourceRange.getLength(),
                                                                              true);
        }
    }
    
    ~TrackSegmentSource() {
        
    }
    
    void prepareToPlay (int samplesPerBlockExpected,
                        double sampleRate) override
    {
        // xx
    }
    
    void releaseResources() override {
        // xx
    }
    
    void getNextAudioBlock (const juce::AudioSourceChannelInfo& bufferToFill) override {
        if (targetRange.contains (position)) {
            auto readerPosition = (position - targetRange.getStart());
            subsectionReader->readSamples ((int* const*) bufferToFill.buffer->getArrayOfWritePointers(),
                                           bufferToFill.buffer->getNumChannels(),
                                           0,
                                           readerPosition,
                                           bufferToFill.numSamples);
        }
        
        position += bufferToFill.numSamples;
    }
    
    void setNextReadPosition (juce::int64 newPosition) override {
        position = newPosition;
    }
    
    juce::int64 getNextReadPosition() const override {
        return position;
    }
    
    juce::int64 getTotalLength() const override {
        return static_cast<juce::int64> (totalRange.getLength());
    }
    
    bool isLooping() const override {
        return false;
    }
    
    double getSourceSampleRate() const { return sourceSampleRate; };
    
private:
    double sourceSampleRate = 0;
    
    juce::Range<juce::int64> totalRange;
    juce::Range<juce::int64> targetRange;
    juce::Range<juce::int64> sourceRange;
    
    juce::int64 position = 0;

    std::unique_ptr<juce::AudioSubsectionReader> subsectionReader;
};

MainComponent::MainComponent() {
    setSize (800, 600);

    if (juce::RuntimePermissions::isRequired (juce::RuntimePermissions::recordAudio)
        && ! juce::RuntimePermissions::isGranted (juce::RuntimePermissions::recordAudio))
    {
        juce::RuntimePermissions::request (juce::RuntimePermissions::recordAudio,
                                           [&] (bool granted) { setAudioChannels (granted ? 2 : 0, 2); });
    }
    else {
        setAudioChannels (2, 2);
    }
}

MainComponent::~MainComponent() {
    shutdownAudio();
}

void MainComponent::prepareToPlay (int samplesPerBlockExpected, double sampleRate)
{
    trackSegmentSource = std::make_unique<TrackSegmentSource>();
    resamplingAudioSource = std::make_unique<juce::ResamplingAudioSource> (trackSegmentSource.get(),
                                                                           false);
    
    const auto sourceSampleRateToCorrectFor = trackSegmentSource->getSourceSampleRate();
    resamplingAudioSource->setResamplingRatio (sourceSampleRateToCorrectFor / sampleRate);
    
    if (trackSegmentSource != nullptr) {
        trackSegmentSource->prepareToPlay (samplesPerBlockExpected, sampleRate);
    }
    
    if (resamplingAudioSource != nullptr) {
        resamplingAudioSource->prepareToPlay (samplesPerBlockExpected, sampleRate);
    }
}

void MainComponent::getNextAudioBlock (const juce::AudioSourceChannelInfo& bufferToFill) {
    bufferToFill.clearActiveBufferRegion();
    
    if (! resamplingEnabled) {
        if (trackSegmentSource != nullptr) {
            trackSegmentSource->getNextAudioBlock (bufferToFill);
        }
    }
    else {
        if (resamplingAudioSource != nullptr) {
            resamplingAudioSource->getNextAudioBlock (bufferToFill);
        }
    }
}

void MainComponent::releaseResources() {
    if (trackSegmentSource != nullptr) {
        trackSegmentSource->releaseResources();
    }
    
    if (resamplingAudioSource != nullptr) {
        resamplingAudioSource->releaseResources();
    }
}

void MainComponent::paint (juce::Graphics& g) {
    g.fillAll (getLookAndFeel().findColour (juce::ResizableWindow::backgroundColourId));
}

void MainComponent::resized() {
    // xx
}
